#/usr/bin/env python
import glob
import os
import shutil
import sys
import tarfile
import subprocess

#configuration steps import
from baseConfigurationStep   import BaseConfigurationStep
from nedeployRCConfig        import NedeployRCConfig
from nedeployInstall         import NedeployInstall
from neadmRCConfig           import NeadmRCConfig
from neadmInitWait           import NeadmInitWait
from neadmSystemInit         import NeadmSystemInit
from neadmLicenseActivation  import NeadmLicenseActivation
from neadmOnlineNodesWait    import NeadmOnlineNodesWait
from neadmClusterCreation    import NeadmClusterCreation
from waitNodeUUID            import WaitNodeUUID

class NedgeBaseConfigurator:
    
    def __init__(self, environment = {}, steps = []):
        self.environment = environment
        self.steps = steps

    def configure(self):
        print('Configuration started')

        try:
            for step in self.steps:
                if isinstance(step, BaseConfigurationStep):
                    print("Processing {} step".format(step.__class__.__name__))
                    #configuration step virtual method
                    step.process(self.environment)
                else:
                    print('WARNING: There is unknown object in configuration steps!')

            return True
        except Exception as e:
            print("Neadm configuration failed. Terminating")
            print('Exception in {}'.format(e.message))
            return False

class NedgeNodeConfigurator(NedgeBaseConfigurator):
    _steps = [
                NedeployRCConfig(),
                NedeployInstall(),
                WaitNodeUUID()
            ]
    def __init__(self, environment = {}):
        environment['node_type'] = 'data'
        NedgeBaseConfigurator.__init__(self, environment, NedgeNodeConfigurator._steps)

class NedgeGatewayConfigurator(NedgeBaseConfigurator):
    _steps = [
                NedeployRCConfig(),
                NedeployInstall(),
                WaitNodeUUID()
            ]
    def __init__(self, environment = {}):
        environment['node_type'] = 'gateway'
        NedgeBaseConfigurator.__init__(self, environment, NedgeNodeConfigurator._steps)

class NedgeMgmtConfigurator(NedgeBaseConfigurator):
    _steps = [
                NedeployRCConfig(),
                NedeployInstall(),
                NeadmRCConfig(),
                NeadmInitWait(),
                NeadmSystemInit(),
                NeadmLicenseActivation(),
                NeadmOnlineNodesWait(),
                NeadmClusterCreation(),
                WaitNodeUUID()
             ]
    def __init__(self, environment = {}):
        environment['node_type'] = 'mgmt'
        NedgeBaseConfigurator.__init__(self, environment, NedgeMgmtConfigurator._steps)

