import sys
import os
import glob
import shutil
import tarfile
import subprocess
import ast

from settings import Settings

#import from charmhelpors folder
parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
os.sys.path.insert(0,parentdir)
from charmhelpers.core.host import (
    umount,
    mkdir,
    cmp_pkgrevno,
    adduser,
    add_user_to_group,
    service_restart
)

from charmhelpers.core.templating import (
    render
)
os.sys.path.remove(parentdir)

#returns NEDGE server UID
def get_sid():

    if os.path.isfile(Settings.SERVERID_CACHE_PATH):
        of = open(Settings.SERVERID_CACHE_PATH, "r")
        line = of.readline()
        of.close()
        if line:
            #need to check sid by regex 
            return line.strip()

    return None


def configure_user(username='nexenta', password='$1$kh4S5TnK$0NKxf09T3TNj.W3ejTgGT1'):
    adduser(username, password, shell='/bin/bash')    
    add_user_to_group(username, 'sudo') 

    #configure SSH for users
    subprocess.call(['sed', '-i', '-e', 's/^.*PasswordAuthentication.*/PasswordAuthentication yes/g', '/etc/ssh/sshd_config'])    
    subprocess.call(['sed', '-i', '-e', 's/^.*PermitRootLogin.*/PermitRootLogin yes/g', '/etc/ssh/sshd_config'])

    #restart ssh service
    service_restart('ssh')


def if_up(ifname):
     subprocess.call(['ifup', ifname])


def configure_replicast_interface(ifname='eth1'):
    
    replicast_eth_file = '/etc/network/interfaces.d/{}.cfg'.format(ifname)
    replicast_if_context = { 'interface_name':ifname }
    render('replicast.conf', replicast_eth_file, replicast_if_context)
    if_up(ifname)


def neadm_bundles_copy():

    NEDEPLOY_FILE_NAME  = "nedeploy-linux_{0}-{1}_x64.tar.gz".format(Settings.NEDGE_BUILD_VERSION, Settings.NEDGE_BUILD_NUMBER)
    NEADM_FILE_NAME     = "neadm-linux_{0}-{1}_x64.tar.gz".format(Settings.NEDGE_BUILD_VERSION, Settings.NEDGE_BUILD_NUMBER)

    NEDEPLOY_REPO_PATH  = "http://10.3.30.163/nedge-dev/nedeploy/{0}/{1}".format(Settings.NEDGE_BUILD_NUMBER, NEDEPLOY_FILE_NAME)
    NEADM_REPO_PATH     = "http://10.3.30.163/nedge-dev/neadm/{0}/{1}".format(Settings.NEDGE_BUILD_NUMBER, NEADM_FILE_NAME)

    nedeploy_tar_file  = 'nedeploy-linux_1.0.0_x64.tar.gz'
    neadm_tar_file     = 'neadm-linux_1.0.0_x64.tar.gz'

    #neadm_charm_path    = os.path.join('files/neadm/', neadm_tar_file)
    #nedeploy_charm_path = os.path.join('files/nedeploy/', nedeploy_tar_file)
    install_dir = '/opt/nedge'

    #create dest folder
    if not os.path.exists(install_dir):
        os.makedirs(install_dir)

    _wget_nedeploy_cmd  = ['wget', '-q', NEDEPLOY_REPO_PATH, '-O',  os.path.join(install_dir, nedeploy_tar_file)]
    _wget_neadm_cmd     = ['wget', '-q', NEADM_REPO_PATH,    '-O',  os.path.join(install_dir, neadm_tar_file)]
    
    subprocess.check_output(_wget_nedeploy_cmd)
    subprocess.check_output(_wget_neadm_cmd)

    #copy tar files to destination
    #shutil.copy(neadm_charm_path, install_dir)
    #shutil.copy(nedeploy_charm_path, install_dir)

    #untar archives
    neadm_tar     = tarfile.open(os.path.join(install_dir, neadm_tar_file))
    nedeploy_tar  = tarfile.open(os.path.join(install_dir, nedeploy_tar_file)) 

    neadm_tar.extractall(install_dir)
    nedeploy_tar.extractall(install_dir)

    #remove archives
    os.remove(os.path.join(install_dir, neadm_tar_file))
    os.remove(os.path.join(install_dir, nedeploy_tar_file))

def install_iscsi_driver():
    cinder_driver_dst = '/usr/lib/python2.7/dist-packages/cinder/volume/drivers/nexenta'
    nexenta_edge_folder = os.path.join(cinder_driver_dst, 'nexentaedge')

    driver_module_file_repo = 'https://raw.githubusercontent.com/Nexenta/cinder/master/cinder/volume/drivers/nexenta/__init__.py'
    driver_repo_src = [
                        'https://raw.githubusercontent.com/Nexenta/cinder/master/cinder/volume/drivers/nexenta/nexentaedge/iscsi.py',
                        'https://raw.githubusercontent.com/Nexenta/cinder/master/cinder/volume/drivers/nexenta/nexentaedge/jsonrpc.py',
                        'https://raw.githubusercontent.com/Nexenta/cinder/master/cinder/volume/drivers/nexenta/nexentaedge/__init__.py'
                      ]
    if not os.path.exists(nexenta_edge_folder):
        os.makedirs(nexenta_edge_folder)

    for src in driver_repo_src:
        wget_cmd = ['wget', '-q', src, '-P', nexenta_edge_folder]
        subprocess.check_output(wget_cmd)

    wget_cmd = ['wget', '-q', driver_module_file_repo, '-P', cinder_driver_dst]
    subprocess.check_output(wget_cmd)
