#!/usr/bin/env python
import glob
import os
import shutil
import sys
import tarfile
import subprocess

from charmhelpers.core import unitdata
from charmhelpers.core.hookenv import (
    log,
    ERROR,
    config,
    relation_ids,
    relation_id,
    related_units,
    relation_get,
    relation_set,
    remote_unit,
    remote_service_name,
    Hooks,
    is_leader,
    UnregisteredHookError,
    service_name,
    unit_public_ip,
    relation_clear,
    is_relation_made,
    local_unit
)

from charmhelpers.core.host import (
    umount,
    mkdir,
    cmp_pkgrevno,
    adduser, 
    add_user_to_group,
    service_restart
)

from charmhelpers.core.templating import (
    render
)

from charmhelpers.contrib.openstack.ip import (
    canonical_url,
    PUBLIC, INTERNAL, ADMIN,
)

from configurationSteps.nedgeConfigurator import (
    NedgeBaseConfigurator,
    NedgeGatewayConfigurator
)
from configurationSteps.settings import Settings
from configurationSteps.utils import (
    get_sid,
    configure_user,
    if_up,
    configure_replicast_interface,
    neadm_bundles_copy
)

hooks = Hooks()
hook_data = unitdata.HookData()
db = unitdata.kv()

@hooks.hook('install')
def install():
    neadm_bundles_copy()
    configure_user()
    log('Bundles has copied')
    replicast_eth = config('replicast_eth')
    configure_replicast_interface(replicast_eth)

    public_ip = unit_public_ip()

    log('replicast interface {} has been created'.format(replicast_eth))
    log('Node public ip is {}'.format(public_ip))
    environment = {  
        'node_public_ip'    :  public_ip,
        'replicast_eth'     :  replicast_eth,
        'profile'             : config('profile'),
        'nodocker'            : config('nodocker'),
        'exclude'             : config('exclude'),
        'reserved'            : config('reserved')
       }

    configurator = NedgeGatewayConfigurator(environment)
    if not configurator.configure():
        log('NEDGE swift gateway node configuration failed.', level=ERROR)
        sys.exit(1)
    return 

@hooks.hook('swift-gw-relation-joined')
def on_mgmt_joined():

    #send node sid to mgmt to create new service
    data = {'gw_info' : {'sid':get_sid(), 'unit_id':local_unit(), 'service_name':service_name()}}
    log("send gw_info:{} to MGMT".format(data))
    relation_set(relation_settings=data)

@hooks.hook('swift-gw-relation-departed')
def on_mgmt_departed():
    #mark service as detached
    context = db.get('swift-context', record=True)
    log("context data is {}".format(context))
    if context:
        context['state'] = 'detached'
        db.set('swift-context', context)

@hooks.hook('swift-gw-relation-changed')
def on_swift_gateway_data_changed():
    #recive and parse message from mgmt
    context =  relation_get('swift-context')
    log("data recived from MGMT: {}".format(context))
    if context:
        context = ast.literal_eval(context)
        if context['sid'] == get_sid() and context['state'] == 'incomplete':

            port    = Settings.KEYSTONE_PORT
            context['operator-roles'] = config('operator-roles')
            context['region']         = config('region')
            context['public_url']     = "{0}:{1}/v1/AUTH_%(tenant_id)s".format(canonical_url(None, PUBLIC), port)
            context['internal_url']   = "{0}:{1}/v1/AUTH_%(tenant_id)s".format(canonical_url(None, INTERNAL), port)
            context['admin_url']      = "{0}:{1}".format(canonical_url(None, ADMIN), port)
            context['state'] = 'complete'

            db.set('swift-context', context)
            log("Swift context is:\n{}".format(context))

            for relid in relation_ids('identity-service'):
                identity_service_joined(relid)
        else:
            log("WRONG data: {}".format(context))


@hooks.hook('identity-service-relation-joined')
def identity_service_joined(relid=None):
    log("keysone identity service joined")    
    context = db.get('swift-context', record=True)
    if context and context['state'] == 'complete':
        #send swift info to keystone
        
        log("send swift-context:{} to keystone".format(context))
        public_url      = context['public_url']
        internal_url    = context['internal_url']
        admin_url       = context['admin_url']
        operator_roles  = context['operator-roles']
        region          = context['region']
        
        relation_set(   service='swift',
                        region=region,
                        public_url=public_url, 
                        internal_url=internal_url,
                        admin_url=admin_url,
                        requested_roles=operator_roles,
                        relation_id=relid)

    else:
        log("[GW] No compete cinder-context to send to cinder-nedge")

    # send data to keystone 

if __name__ == '__main__':
    try:
        with hook_data():
            hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))

